﻿//
// Copyright (C) 2011, ZAO Nanosoft. All rights reserved.
//
namespace HostMgdReversi
{
  using System;
  using System.Collections.Generic;
  using System.Linq;
  using System.Text;

#if NCAD
  using HostMgd.ApplicationServices;
  using HostMgd.EditorInput;

  using Teigha.DatabaseServices;
  using Teigha.Runtime;
  using Teigha.Geometry;

  using Platform = Teigha;
  using Host = HostMgd;
#else
    using Autodesk.AutoCAD.DatabaseServices;
    using Autodesk.AutoCAD.Runtime;
    using Autodesk.AutoCAD.Geometry;
    using Autodesk.AutoCAD.ApplicationServices;
    using Autodesk.AutoCAD.EditorInput;
    
    using Platform = Autodesk.AutoCAD;
    using Host = Autodesk.AutoCAD;
#endif

  class Game
  {
    Desk gamedesk = new Desk();
    ScorePanel panel = new ScorePanel();

    ePlayer currPlayer = ePlayer.Human;

    public void Init()
    {            
      gamedesk = new Desk();
      gamedesk.Init();
      gamedesk.DrawDesk();

      panel = new ScorePanel();
      panel.Init();
      panel.DrawPanel();
    }

    public void Play()
    {
      while (CanMove(ePlayer.Human) || CanMove(ePlayer.PC))
      {
        DrawScore();
        if (currPlayer == ePlayer.Human)
        {
          if (PlayerTurn() == TurnResult.Cancel)
            break;
        }
        else
        {
          AITurn();
        }
        currPlayer = (currPlayer == ePlayer.Human) ? ePlayer.PC : ePlayer.Human;
      }

      currPlayer = ePlayer.Empty;
      DrawScore();
    }

    ObjectId WaitPlayerAction()
    {
      Editor ed = Application.DocumentManager.MdiActiveDocument.Editor;

      ObjectId selectObj = ObjectId.Null;
      PromptEntityOptions opts = new PromptEntityOptions("Your move. Select cell");
      opts.SetRejectMessage("\nYou should select cell only.");
      opts.AddAllowedClass(typeof(Hatch), false);

      while (selectObj == ObjectId.Null)
      {
        PromptEntityResult pr = ed.GetEntity(opts);

        if (pr.Status == PromptStatus.Cancel)
          return ObjectId.Null;

        if (!gamedesk.GameDesc[pr.ObjectId].IsBusy)
        {
          // Only free cells can be selected
          selectObj = pr.ObjectId;
        }
      }
      return selectObj;
    }

    TurnResult PlayerTurn()
    {
      // if no cell to make move - exit
      if (!CanMove(ePlayer.Human))
      {
        return TurnResult.Unable;
      }

      bool turnComplite = false;
      while (!turnComplite)
      {
        ObjectId objId = WaitPlayerAction();
        if (objId == ObjectId.Null)
        {
          return TurnResult.Cancel;
        }

        var piece = gamedesk.GameDesc[objId];
        if (CanDo(piece.x, piece.y, currPlayer, true) > 0)
        {
          gamedesk.CreatePiece(piece.x, piece.y, currPlayer);
          turnComplite = true;
        }
      }
      return TurnResult.Ok;
    }

    TurnResult AITurn()
    {
      byte[,] table = {
        {1, 8, 2, 2, 2, 2, 8, 1},
        {8, 8, 6, 5, 5, 6, 8, 8},
        {2, 6, 4, 3, 3, 4, 6, 2},
        {2, 5, 3, 1, 1, 3, 5, 2},
        {2, 5, 3, 1, 1, 3, 5, 2},
        {2, 6, 4, 3, 3, 4, 6, 2},
        {8, 8, 6, 5, 5, 6, 8, 8},
        {1, 8, 2, 2, 2, 2, 8, 1} };

      var turns = from c in this.gamedesk.GameDesc
                  let cell = c.Value
                  let count = CanDo(c.Value.x, c.Value.y, ePlayer.PC, false)
                  where (cell.IsBusy == false) && (count > 0)
                  orderby count descending
                  orderby table[cell.x, cell.y]
                  select cell;

      if (turns.Count() > 0)
      {
        // make move
        Piece piece = turns.ElementAt(0);
        CanDo(piece.x, piece.y, ePlayer.PC, true);
        gamedesk.CreatePiece(piece.x, piece.y, ePlayer.PC);
        return TurnResult.Ok;
      }
      else
      {
        // if it is impossible to move - exit
        return TurnResult.Unable;
      }
    }

    bool CanMove(ePlayer player)
    {
      foreach (var piece in (from c in gamedesk.GameDesc where !c.Value.IsBusy select c.Value))
      {
        if (CanDo(piece.x, piece.y, player, false) > 0)
          return true;
      }
      return false;
    }

    int CanDo(int col, int row, ePlayer player, bool dostep)
    {
      // If cell is occupied then it can move here
      if (gamedesk.GameDesc_xy[col, row].IsBusy)
      {
        return 0;
      }

      int total = 0;
      ePlayer opponent = (player == ePlayer.Human) ? ePlayer.PC : ePlayer.Human;

      total += CheckDirection(col, row, eDirection.Up,    opponent, dostep);
      total += CheckDirection(col, row, eDirection.Down,  opponent, dostep);
      total += CheckDirection(col, row, eDirection.Left,  opponent, dostep);
      total += CheckDirection(col, row, eDirection.Right, opponent, dostep);

      total += CheckDirection(col, row, eDirection.Up   | eDirection.Left,  opponent, dostep);
      total += CheckDirection(col, row, eDirection.Up   | eDirection.Right, opponent, dostep);
      total += CheckDirection(col, row, eDirection.Down | eDirection.Left,  opponent, dostep);
      total += CheckDirection(col, row, eDirection.Down | eDirection.Right, opponent, dostep);

      return total;
    }

    int CheckDirection(int x, int y, eDirection direction, ePlayer opponent, bool dostep)
    {
      short dx = 0;
      short dy = 0;
      if ((direction & eDirection.Down) == eDirection.Down) dy -= 1;
      if ((direction & eDirection.Up) == eDirection.Up) dy += 1;
      if ((direction & eDirection.Left) == eDirection.Left) dx -= 1;
      if ((direction & eDirection.Right) == eDirection.Right) dx += 1;

      if ((dx == 0) && (dy == 0)) return 0;

      ePlayer player = (opponent == ePlayer.Human) ? ePlayer.PC : ePlayer.Human;

      int cx = x + dx;
      int cy = y + dy;
      while ((cx < 8) && (cx >= 0) && (cy < 8) && (cy >= 0))
      {
        int l = Math.Max(Math.Abs(x - cx), Math.Abs(y - cy)) - 1;
        if (gamedesk.GameDesc_xy[cx, cy].Player == player)
        {
          if (dostep)
            for (int i = 1; i <= l; ++i)
            {
              gamedesk.GameDesc_xy[x + i * dx, y + i * dy].Player = player;
            }

          return l;
        }
        if (gamedesk.GameDesc_xy[cx, cy].Player == ePlayer.Empty)
        {
          return 0;
        }
        cx += dx;
        cy += dy;
      }
      return 0;
    }

    int GetCounterCount(ePlayer player)
    {
      return gamedesk.GameDesc.Where(x => x.Value.Player == player).Count();
    }

    void DrawScore()
    {
      int wc = GetCounterCount(ePlayer.Human);
      int bc = GetCounterCount(ePlayer.PC);
      panel.WhiteScore = wc.ToString();
      panel.BlackScore = bc.ToString();

      if (currPlayer != ePlayer.Empty)
      {
        panel.CurrentAction = (currPlayer == ePlayer.Human) ? "Blue move" : "Black move";
        panel.Leader = (wc > bc) ? "Blue is winning" : (wc < bc) ? "Black is winning" : "Draw for a while";
      }
      else
      {
        panel.CurrentAction = "";
        panel.Leader = (wc > bc) ? "Blue wins" : (wc < bc) ? "Black wins" : "Draw";
      }
    }

  }
}
