﻿//
// Copyright (C) 2011, ZAO Nanosoft. All rights reserved.
//
namespace HostMgdReversi
{
  using System;
  using System.Collections.Generic;
  using System.Linq;
  using System.Text;
    
  #if NCAD
    using HostMgd.ApplicationServices;
    using Teigha.DatabaseServices;
    using Teigha.Geometry;
    
    using Platform = Teigha;
  #else
    using Autodesk.AutoCAD.DatabaseServices;
    using Autodesk.AutoCAD.Geometry;
    using Autodesk.AutoCAD.ApplicationServices;
    
    using Platform = Autodesk.AutoCAD;
  #endif
  /// <summary>
  /// Piece Class (Colored disk)
  /// </summary>
  class Piece
  {
    #region Variables
    Database db = null;

    ObjectId pieceId = ObjectId.Null;
    public int x = -1;
    public int y = -1;
    ePlayer player = ePlayer.Empty;
    #endregion

    public Piece(int x, int y)
    {
      this.db = Application.DocumentManager.MdiActiveDocument.Database;

      this.x = x;
      this.y = y;
    }
    /// <summary>
    /// Returns true if cell is occupied
    /// </summary>
    public Boolean IsBusy
    {
      get
      { 
        return (this.pieceId != ObjectId.Null); 
      }
    }

    /// <summary>
    /// Property of cell owner (Player)
    /// </summary>
    public ePlayer Player
    {
      set
      {
        if (value == this.player)
        {
          return;
        }

        if (value == ePlayer.Empty)
        {
          throw new ArgumentException("Player property can not be set to Empty");
        }

        this.player = value;
        if (ObjectId.Null != this.pieceId)
        {
          using (Transaction myT = db.TransactionManager.StartTransaction())
          {
            var piece = myT.GetObject(this.pieceId, OpenMode.ForWrite) as PolygonMesh;
            piece.Color = (player == ePlayer.Human) ? Constants.HumanColor : Constants.PcColor; 

            myT.Commit();
          }
        }
      }
      get
      {
        return player;
      }
    }

    public override string ToString()
    {
      return string.Format("X: {0}, Y: {1}, Player: {2}", this.x, this.y, this.player);
    }
    
    /// <summary>
    /// Create disk (piece) on Game Board 
    /// </summary>
    public void DrawPiece()
    {
      using (Transaction myT = db.TransactionManager.StartTransaction())
      {
        BlockTable bt = myT.GetObject(db.BlockTableId, OpenMode.ForRead, false) as BlockTable;
        BlockTableRecord ms = myT.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false) as BlockTableRecord;

        this.DrawPiece(ms, myT);

        myT.Commit();
      }
    }
    public void DrawPiece(BlockTableRecord btr, Transaction tr)
    {
      double alfa, beta;
      double X, Y, Z;

      PolygonMesh mesh = null;
      mesh = new PolygonMesh();

      mesh.NSize = 20;
      mesh.MSize = 20;
      mesh.MakeNClosed();
      mesh.SetDatabaseDefaults();
      this.pieceId = btr.AppendEntity(mesh);
      tr.AddNewlyCreatedDBObject(mesh, true);

      for (int j = 0; j <= 20; ++j)
      {
        for (int i = 0; i < 20; ++i)
        {
          beta = Math.PI / 19 * j;
          alfa = 2 * Math.PI / 20 * i;
          X = (this.x * Constants.gridstep) + Constants.gridstep / 2 + Constants.counter_radius * (Math.Cos(alfa) * Math.Sin(beta));
          Y = (this.y * Constants.gridstep) + Constants.gridstep / 2 + Constants.counter_radius * (Math.Sin(alfa) * Math.Sin(beta));
          Z = Constants.counter_radius * Math.Cos(beta) * (0.5 * Math.Sin(beta) + 0.5);
          AddVertexToMesh(mesh, new Point3d(X, Y, Z), tr);
        }
      }
    }

    /// <summary>
    /// Draw Cell background on Game Board
    /// </summary>
    public void DrawCell()
    {
      using (Transaction myT = db.TransactionManager.StartTransaction())
      {
        BlockTable bt = myT.GetObject(db.BlockTableId, OpenMode.ForRead, false) as BlockTable;
        BlockTableRecord ms = myT.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false) as BlockTableRecord;

        this.DrawCell(ms, myT);

        myT.Commit();
      }
    }
    public ObjectId DrawCell(BlockTableRecord ms, Transaction tr)
    {
      /// Create closed polyline - hatch border
      Polyline pln = new Polyline(4);

      Point2d pt1 = new Point2d(this.x * Constants.gridstep, this.y * Constants.gridstep);
      Point2d pt2 = new Point2d(this.x * Constants.gridstep, this.y * Constants.gridstep + Constants.gridstep);
      Point2d pt3 = new Point2d(this.x * Constants.gridstep + Constants.gridstep, this.y * Constants.gridstep + Constants.gridstep);
      Point2d pt4 = new Point2d(this.x * Constants.gridstep + Constants.gridstep, this.y * Constants.gridstep);

      pln.AddVertexAt(0, pt1, 0, 0, 0);
      pln.AddVertexAt(1, pt2, 0, 0, 0);
      pln.AddVertexAt(2, pt3, 0, 0, 0);
      pln.AddVertexAt(3, pt4, 0, 0, 0);
      pln.Closed = true;

      /// Add border to model space
      ObjectId pid = ms.AppendEntity(pln);
      tr.AddNewlyCreatedDBObject(pln, true);

      /// Create hatch 
      Hatch hatch = new Hatch();
      ObjectId hid = ms.AppendEntity(hatch);
      hatch.SetDatabaseDefaults();
      tr.AddNewlyCreatedDBObject(hatch, true);

      hatch.SetHatchPattern(HatchPatternType.PreDefined, "SOLID");
      hatch.Color = Constants.CellColor;

      ObjectIdCollection objIdCol = new ObjectIdCollection();
      objIdCol.Add(pid);

      hatch.AppendLoop(HatchLoopTypes.Default, objIdCol);
      hatch.EvaluateHatch(true);

      return hatch.ObjectId;
    }

    /// <summary>
    /// Add vertex to 3D mesh node and to the transaction
    /// </summary>
    /// <param name="polymech">3D mesh to add vertex</param>
    /// <param name="pt3d">vertex to be added</param>
    /// <param name="tr">Mesh modification transaction</param>
    private void AddVertexToMesh(PolygonMesh polymech, Point3d pt3d, Transaction tr)
    {
      PolygonMeshVertex acPMeshVer = new PolygonMeshVertex(pt3d);
      polymech.AppendVertex(acPMeshVer);
      tr.AddNewlyCreatedDBObject(acPMeshVer, true);
    }
  }
}
